/**
 * @(#)sidt2.h 2002/02/25
 *
 * @author Ice
 * @version 1.00
 *
 * ICE Team free software group
 *
 * This file is part of TSID2.
 * See README for copyright notice.
 *
 * note: some comments are in the style of Javadoc of Java language!
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
 *  02111-1307  USA.
 */

#ifndef _SIDT2_H
#define _SIDT2_H

#include "version2.h"
#include <libini.h>

#define MD5_EXT ".dat"
#define MD5_EXT_LEN 4

#define MAX_SONGS 128

// show debug string if DEBUG is defined
#ifdef DEBUG_SIDT2
  #define PDEBUG2(a) \
    cerr << "[SIDT2] " << a;
#else
  #define PDEBUG2(a) ;
#endif

/**
 * SIDT2 class
 *
 * This class manage a SIDT2 file.
 *
 * A SIDT2 files is a INI files that contains:
 * <pre>
 *   [STATS]
 *   1=time listen length
 *   ..
 *   n=time listen length
 *
 *   [TUNE]
 *   path=/path_of_file/
 *   name=name_of_file.sid
 * </pre>
 *
 * For speed up a sid player, the methods read must be called only before a
 * writing, because read add the readed information to the current in memory
 * (usually the operation should be the inverse: read from file, then add new
 * information and finally write them).
 * Also, for speed up, we use the old SIDT vector for storing the information
 * and only <code>read</code> and <code>write</code> go to the INI files with
 * libini. This is probably more quick than use directly libini at each sidt
 * operation.
 *
 * @author ice
 * @version 1.00 25/02/2002
 */
class SIDT2 {
  public:
    // Enum that describes the possible errors this class may encounter.
    enum SIDT2error {
      NO_SIDT2_ERROR = 0,   // no error
      NO_MEMORY,            // error: no more memory
      NO_MD5_ERROR,         // error: no MD5 for this song
      NO_FILE_ERROR,        // error: no file path for this song
      NO_BASE_ERROR,        // error: no base path for the stored information
      INI_OPEN_ERROR,       // error opening the INI database
      INI_DELIMS_ERROR      // error setting delims in the database
    };

    // a SIDT2 record of information
    struct sidtRecord {
      unsigned int tune;             // subtune number
      unsigned long second;          // listen second
      unsigned long number;          // number of listen time
      unsigned long length;          // length of the tune if known or 0
    };

    typedef sidtRecord sidtInfo[MAX_SONGS];// sidtInfo;

    /**
     * Construnct SIDT2 using the passed information
     * All the passed path separator must be as ANSI
     * The special absSidFileName #/# is to be used when you did want
     * to save the old file name
     *
     * @param MD5 the MD5 fingerprinter of this file
     * @param absSidFileName the absolute sid file path and name
     * @param absBasePath the base path where there are the sidt2 files
     */
    SIDT2(const char *MD5, const char *absSidFileName, const char *absBasePath);

    /**
     * Destruct the SIDT2
     */
    ~SIDT2();

    /**
     * Return the state of SIDT: false means that SIDT cannot continue
     */
    operator bool() {
      return sidtStatus;
    }

    /**
     * Get the code of the last error
     */
    inline SIDT2error getCodeError() {
      return lastError;
    }

    /**
     * Get Version of TSID Package
     */
    static inline const char *getTSID2Version() {
      return TSID2_VERSION;
    }

    /**
     * Get the error string of the last error
     *
     * @return the string of the error
     */
    const char *getError();

    /**
     * Get the version of SIDT as a formatted string
     *
     * @return the version of SIDT
     */
    inline char *getVersion() {
      return versionString;
    };

    /**
     * Add the passed information to the vector. The vector will be ordinate.
     *
     * @param tune the number of sub-tune to update or add
     * @param second the number of second to add
     * @param number the number of time to add (default=1)
     * @param length the length of the tune (default=0)
     */
    void add(unsigned int tune, unsigned long second, unsigned long number=1, unsigned long length=0);

    /**
     * Add the passed struct of information to the vector.
     * The vector will be ordinate.
     *
     * @param rec the record of information
     */
    void add(struct sidtRecord rec);

    /**
     * Read the record from a file if it's present. The readed record id add to
     * the actual record.
     *
     * @return true if operation is ok
     */
    bool read();

    /**
     * Write the sidt records in the file
     *
     * @return true if operation ok
     */
    bool write();

    /**
     * Shift the sub tunes from one by some step
     * This is used when a new sub tune is added in a sid file
     *
     * @param step the shift step
     * @param from the number of subtunes from being to shift
     */
    void shift(unsigned int step, unsigned int from);

    /**
     * Swap the position of two tunes
     *
     * @param tune1 the first tune
     * @param tune2 the second tune
     */
    void swap(unsigned int song1, unsigned int song2);

    /**
     * Remove the field for the passed tune from memory
     *
     * @param tune the number of tune to erase
     */
    void remove(unsigned int tune);

    /**
     * Init the engin for getting the next tune information
     */
    void initForGet();

    /**
     * Get the next available tune information from the sidt2 record
     *
     * @param tune the returned tune number
     * @param second the returned number of seconds
     * @param number the returned number of time
     * @param length the returned tune length (0=unknown)
     * @return true if the record is present
     */
    bool getNext(unsigned int *tune, unsigned long *second,
                 unsigned long *number, unsigned long *length);

    /**
     * Get the next available information as a record
     *
     * @param rec the returned record of information
     * @return true if the record is present
     */
    bool getNext(struct sidtRecord *rec);


    /**
     * Get the information for the passed tune if present
     *
     * @param tune the tune to look for
     * @param rec the returned record of information
     * @return true if the tune is present
     */
    bool getTune(unsigned int tune, struct sidtRecord *rec);

    /**
     * Get a copy of the readed sid file path
     *
     * @return a copy of the sid file path
     */
    char *getReadSidPath();

    /**
     * Get a copy of the readed sid file name
     *
     * @return a copy of the sid file name
     */
    char *getReadSidName();
    
    /**
     * Get a copy of the MD5 fingerprinter of this file
     *
     * @return a copy of the MD5 of this file
     */
    char *getMD5();

  private:
    // the sidt info of one file
    sidtInfo sidt;

    // Error number of the last error that happened.
    SIDT2error lastError;

    // Version number/copyright string
    char *versionString;

    // MD5 fingerprinter of this files
    char *MD5;

    // Absolute sid file path and name
    char *absSidFileName;

    // The absolute base path where there are the sidt2 files
    char *absBasePath;

    // The complete file name of the MD5 file
    char *absFileName;

    // The old read absolute path of sid file
    char *oldAbsSidPath;

    // The old read name of the sid file
    char *oldSidName;

    // true if sidt2 can continue to work correctly
    bool sidtStatus;

    // index for next record to obtain from getNext
    int sidtIndex;

    // INI datadase of the sidt2 file
    ini_fd_t database;
};

#endif // _SIDT2_H
